<?php

require_once 'custom/PxRoutines/LogicBuilderPackages/lb5bd0806c702871_50528271/operators/FlowchartOperator_lb5bd0806c702871_50528271.php';

class RetrieveEntitiesByAttributesOperator_lb5bd0806c702871_50528271 extends FlowchartOperator_lb5bd0806c702871_50528271{

    protected $entityName;
    protected $conditions;

    protected $detectedEntitiesIds = array();
    protected $detectedEntities = array();

    public function __construct($params){
        parent::__construct($params);
    }

    public function setInDataPorts($params){
        $this->entityName = $params['inDataPorts']['module']['value'];

        foreach($params['inDataPorts'] as $portId => $inData){
            $this->inDataPorts[$portId] = $inData;
            if($portId != 'module')
                $this->conditions[$inData['value']] = null;
        }
    }

    public function setInDataPortById($id, $value){
        if($id == 'module')
            $this->entityName = $value;
        else{
            $fieldName = $this->inDataPorts[$id]['value'];
            $this->conditions[$fieldName] = $value;
        }
    }

    public function process($params = array()){
        if(empty($this->entityName)) return;

        $bean = BeanFactory::getBean($this->entityName);

        if(empty($bean)) return;

        try{
            $this->runQuery($bean);
        }catch(Exception $e){}
    }

    /**
     * @param $bean
     * @throws SugarQueryException
     */
    protected function runQuery(SugarBean $bean){
        $query = new SugarQuery();
        $query
            ->from($bean, array('team_security' => false))
            ->select(array('id'));

        $processSearch = true;
        foreach($this->conditions as $fieldName => $searchValue)
            if(!empty($bean->field_defs[$fieldName])){
                if($bean->field_defs[$fieldName]['source'] != 'non-db'){
                    if(empty($searchValue)){
                        $query->where()->isEmpty($fieldName);
                    } else{
                        $query->where()->equals($fieldName, $searchValue);
                    }
                }elseif(!empty($bean->field_defs[$fieldName]['link'])){
                    if($bean->field_defs[$fieldName]['type'] == 'email'){
                        if(filter_var($searchValue, FILTER_VALIDATE_EMAIL)){
                            $searchValue = strtoupper($searchValue);
                            $sql = "
                            SELECT {$bean->getTableName()}.id id
                            FROM {$bean->getTableName()}
                            INNER JOIN email_addr_bean_rel jt1_email_addr_bean_rel ON (
                                {$bean->getTableName()}.id = jt1_email_addr_bean_rel.bean_id
                                AND jt1_email_addr_bean_rel.deleted = 0
                                AND jt1_email_addr_bean_rel.bean_module = '{$bean->getModuleName()}'
                            )
                            INNER JOIN email_addresses jt0_email_addresses ON (
                                jt0_email_addresses.id = jt1_email_addr_bean_rel.email_address_id
                                AND jt0_email_addresses.deleted = 0
                            )
                            WHERE {$bean->getTableName()}.deleted = 0
                                AND jt0_email_addresses.email_address_caps = '$searchValue'
                        ";
                            $query->where()->addRaw(' ' . $bean->getTableName() . '.id in (' . $sql . ') ');
                        }else{
                            $processSearch = false;
                        }
                    }else{
                        $searchLink = $bean->field_defs[$fieldName]['link'];
                        $searchJoin = $query->join($searchLink, array('joinType' => 'INNER'));
                        $searchRelateField = $bean->field_defs[$fieldName]['rname'];
                        if(empty($searchValue))
                            $query->where()->isEmpty($searchJoin->joinName() . '.' . $searchRelateField);
                        else
                            $query->where()->equals($searchJoin->joinName() . '.' . $searchRelateField, $searchValue);
                    }
                }
                else{
                    $processSearch = false;
                }
            }else{
                $processSearch = false;
            }
        if($processSearch)
            foreach($query->execute() as $queryResult){
                $this->detectedEntitiesIds[$queryResult['id']] = $queryResult['id'];
            }
    }

    public function getOutDataByPortId($portId){
        if($portId == 'entities'){
            return $this->getEntities();
        }else if($portId == 'entitiesNumber'){
            return is_array($this->detectedEntitiesIds) ? count($this->detectedEntitiesIds) : 0;
        }else if($portId == 'singleEntity'){
            return $this->getEntity();
        }

        return null;
    }

    protected function getEntities(){
        if(!empty($this->detectedEntitiesIds) &&  count($this->detectedEntities) != count($this->detectedEntitiesIds)){
            foreach($this->detectedEntitiesIds as $id){
                if(empty($this->detectedEntities[$id]))
                    $this->detectedEntities[$id] = BeanFactory::getBean($this->entityName, $id, array('strict_retrieve' => true));
            }
        }

        return $this->detectedEntities;
    }

    protected function getEntity(){
        if(!empty($this->detectedEntitiesIds)){
            if(!empty($this->detectedEntities))
                $randEntityId = array_rand($this->detectedEntities, 1);
            else{
                $randEntityId = array_rand($this->detectedEntitiesIds, 1);
                $this->detectedEntities[$randEntityId] = BeanFactory::getBean($this->entityName, $randEntityId, array('strict_retrieve' => true));
            }

            return $this->detectedEntities[$randEntityId];
        }

        return null;
    }
}
