<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\TimelineViewer\Package\Import;

class LBTemplatesImport{

    private $directory = 'custom/PxRoutines/TimelineViewer/Package/Import/Data';

    private $filename = 'LBTemplates.php';

    private $refreshFlag = true;

    public function setDirectory($directory){
        $this->directory = $directory;
    }

    public function setFilename($filename){
        $this->filename = $filename;
    }

    public function setForce($refreshFlag){
        $this->refreshFlag = $refreshFlag;
    }

    public function import(){
        if(!$this->isLBTemplatesInstalled()){
            $GLOBALS['log']->info('LB Templates module is missing. Import of predefined data cannot proceed.');
            return;
        }
        $data = $this->readData();

        if (count($data) > 0) {
            foreach ($data as $item) {
                $this->importItem($item);
            }
        }
    }

    private function isLBTemplatesInstalled(){
        $lbTemplatesModuleName = 'PX_FlowchartTemplates';
        $lbTemplateBean = \BeanFactory::getBean($lbTemplatesModuleName);

        return !empty($lbTemplateBean);
    }

    public function readData(){
        return $this->loadDataFromFile($this->directory . '/' . $this->filename);
    }

    private function loadDataFromFile($file){
        if(\SugarAutoLoader::fileExists($file)){
            return require $file;
        }else{
            return array();
        }
    }

    private function importItem($item){
        $id = $item['id'];
        $lbTemplateBean = $this->getRecord('PX_FlowchartTemplates', $id);
        $this->syncRecord($lbTemplateBean, $item);
    }

    private function getRecord($moduleName, $id){
        $bean = \BeanFactory::getBean($moduleName, $id, array(), false);

        if (empty($bean->id)) {
            $bean->id = $id;
            $bean->new_with_id = true;
        } elseif(!empty($bean->deleted)) {
            $bean->mark_undeleted($id);
        }

        return $bean;
    }

    private function syncRecord(\SugarBean $bean, array $item){
        if($this->populateData($bean, $item)){
            $bean->save();
        }
    }

    private function populateData(\SugarBean $bean, array $item){
        $changes = false;

        if($this->checkField('name', $bean, $item)){
            $bean->name = $item['name'];
            $changes = true;
        }
        if($this->checkField('tpl_engine_type', $bean, $item)){
            $bean->tpl_engine_type = $item['tpl_engine_type'];
            $changes = true;
        }
        if($this->checkField('context', $bean, $item)){
            $bean->context = $item['context'];
            $changes = true;
        }
        if($this->checkField('html_body', $bean, $item)){
            $bean->html_body = $item['html_body'];
            $changes = true;
        }

        return $changes;
    }

    private function checkField($fieldName, \SugarBean $bean, array $item){
        if (isset($bean->field_defs[$fieldName]) && $bean->field_defs[$fieldName]['type'] != 'link') {
            if($bean->fetched_row[$fieldName] != $item[$fieldName]){
                return true;
            }
        }

        return false;
    }

}