<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\TimelineViewer\Package\Import;

class TimelineEventsImport{

    private $directory = 'custom/PxRoutines/TimelineViewer/Package/Import/Data';

    private $filename = 'TimelineEvents.php';

    private $refreshFlag = true;

    public function setDirectory($directory){
        $this->directory = $directory;
    }

    public function setFilename($filename){
        $this->filename = $filename;
    }

    public function setForce($refreshFlag){
        $this->refreshFlag = $refreshFlag;
    }

    public function import(){
        if(!$this->isTLVEventsInstalled()){
            $GLOBALS['log']->info('Timeline Events module is missing. Import of predefined data cannot proceed.');
            return;
        }
        $data = $this->readData();

        if (count($data) > 0) {
            foreach ($data as $item) {
                $this->importItem($item);
            }
        }
    }

    private function isTLVEventsInstalled(){
        $tlvEventsModuleName = 'PX_TimelineEvents';
        $tlvEventsBean = \BeanFactory::getBean($tlvEventsModuleName);

        return !empty($tlvEventsBean);
    }

    public function readData(){
        return $this->loadDataFromFile($this->directory . '/' . $this->filename);
    }

    private function loadDataFromFile($file){
        if(\SugarAutoLoader::fileExists($file)){
            return require $file;
        }else{
            return array();
        }
    }

    private function importItem($item){
        $id = $item['id'];
        $tlvEvent = $this->getRecord('PX_TimelineEvents', $id);
        $this->syncRecord($tlvEvent, $item);
    }

    private function getRecord($moduleName, $id){
        $bean = \BeanFactory::getBean($moduleName, $id, array(), false);

        if (empty($bean->id)) {
            $bean->id = $id;
            $bean->new_with_id = true;
        } elseif(!empty($bean->deleted)) {
            $bean->mark_undeleted($id);
        }

        return $bean;
    }

    private function syncRecord(\SugarBean $bean, array $item){
        if($this->populateData($bean, $item)){
            $bean->save();
        }
    }

    private function populateData(\SugarBean $bean, array $item){
        $changes = false;

        if($this->checkField('name', $bean, $item)){
            $bean->name = $item['name'];
            $changes = true;
        }
        if($this->checkField('icon', $bean, $item)){
            $bean->icon = $item['icon'];
            $changes = true;
        }
        if($this->checkField('target_module', $bean, $item)){
            $bean->target_module = $item['target_module'];
            $changes = true;
        }
        if($this->checkField('lifetime_units', $bean, $item)){
            $bean->lifetime_units = $item['lifetime_units'];
            $changes = true;
        }
        if($this->checkField('lifetime_duration', $bean, $item)){
            $bean->lifetime_duration = $item['lifetime_duration'];
            $changes = true;
        }

        return $changes;
    }

    private function checkField($fieldName, \SugarBean $bean, array $item){
        if (isset($bean->field_defs[$fieldName]) && $bean->field_defs[$fieldName]['type'] != 'link') {
            if($bean->fetched_row[$fieldName] != $item[$fieldName]){
                return true;
            }
        }

        return false;
    }

}